package org.jboss.soa.esb.samples.quickstart.juddiv3;

import org.apache.juddi.v3.client.config.UDDIClerkManager;
import org.apache.juddi.v3.client.config.UDDIClientContainer;
import org.apache.juddi.v3.client.transport.Transport;

import org.apache.juddi.v3_service.JUDDIApiPortType;

import org.uddi.v3_service.UDDICustodyTransferPortType;
import org.uddi.v3_service.UDDIInquiryPortType;
import org.uddi.v3_service.UDDIPublicationPortType;
import org.uddi.v3_service.UDDISecurityPortType;
import org.uddi.v3_service.UDDISubscriptionListenerPortType;
import org.uddi.v3_service.UDDISubscriptionPortType;

public class JUddiClient {

	Transport transport;
	
	public JUddiClient(String configFile) {
		try {
			UDDIClerkManager manager = new UDDIClerkManager(configFile);
			manager.start();
			
			UDDIClientContainer.addClerkManager(manager);

			String clazz = manager.getClientConfig().getUDDINode("client-ws").getProxyTransport();
			Class<?> transportClass = forName(clazz, Transport.class);
			
			transport = (Transport) transportClass.getConstructor(String.class).newInstance("client-ws");
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	public UDDIPublicationPortType getPublicationPort() throws Exception {
		return transport.getUDDIPublishService();
	}
	
	public UDDIInquiryPortType getInquiryPort() throws Exception {
		return transport.getUDDIInquiryService();
	}
	
	public UDDISecurityPortType getSecurityPort() throws Exception {
		return transport.getUDDISecurityService();
	}
	
	public UDDICustodyTransferPortType getCustodyTransferPort() throws Exception {
		return transport.getUDDICustodyTransferService();
	}
	
	public UDDISubscriptionPortType getSubscriptionPort() throws Exception {
		return transport.getUDDISubscriptionService();
	}
	
	public UDDISubscriptionListenerPortType getSubscriptionListenerPort() throws Exception {
		return transport.getUDDISubscriptionListenerService();
	}
	
	public JUDDIApiPortType getApiPort() throws Exception {
		return transport.getJUDDIApiService();
	}
	
	private Class<?> forName(String name, Class<?> caller) throws ClassNotFoundException {
		ClassLoader threadClassLoader = Thread.currentThread().getContextClassLoader();
		if(threadClassLoader != null) {
			try {
				return Class.forName(name, true, threadClassLoader);
			} catch (ClassNotFoundException e) {
				if(e.getException() != null) {
					throw e;
				}
			}
		}
		
		ClassLoader callerClassLoader = caller.getClassLoader();
		if(callerClassLoader != null) {
			try {
				return Class.forName(name, true, callerClassLoader);
			} catch (ClassNotFoundException e) {
				if(e.getException() != null) {
					throw e;
				}
			}
		}
		
		return Class.forName(name, true, ClassLoader.getSystemClassLoader());
		
	}
	
}
